#!/bin/awk -f
#
#
# Make c code drivers for calling the callbacks used to emit code. There 
# is no way in c++ to link data to function calls within a class hierarchy 
# while using a comming driver. This program writes c-callbacks automatically.

# Match class names.
/^[ \t]*class[ \t][ \t]*[A-Za-z_]+/ {
	curClass = $2
}

# Look for codegen functions in classes only.
$1 == "void" && $2 == "codegen" && curClass != "" {
	split( $3, a, "[ \t()]+");
	funcs[curClass "::" a[1]] = 1;
	classes[curClass] = 1;
	files[FILENAME] = 1;
}

# Look for the end of classes.
/^};/ {
	curClass = "";
}

END {
	# Includes
	print "/*";
	print " * This file is part of ragel. It is automatically";
	print " * generated by mkgendrv.awk.";
	print " */";

	for ( fn in files )
		print "#include \"" fn "\"";

	# Write out the stub functions that call the versions in classes.
	for ( fname in funcs ) {
		print "";
		split( fname, a, "::" );
		print "void " a[1] "_" a[2] "( FsmCodeGen *codeGen )";
		print "{"
		print "\t" a[1] " *super = (" a[1] "*) codeGen;";
		print "\tsuper->" a[2] "();";
		print "}"
	}

	# Write section registering functions.
	for ( c in classes ) {
		print "";
		print "void " c "::registerSections()"
		print "{"
		for ( fname in funcs ) {
			split( fname, a, "::" );
			if ( a[1] == c )
				print "\tregisterSection( \"" a[2] "\", " c "_" a[2] " );"
		}
		print "}"
	}
}
